/*
 * Elaastic - formative assessment system
 * Copyright (C) 2019. University Toulouse 1 Capitole, University Toulouse 3 Paul Sabatier
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package org.elaastic.questions.attachment.datastore

import java.io.File
import java.io.IOException
import java.io.InputStream

/**
 * Data record that is based on a normal file.
 */
class FileDataRecord(
        identifier: DataIdentifier,
        private val file: File
) : AbstractDataRecord(identifier) {

    /**
     * {@inheritDoc}
     */
    override val length: Long
        get() = file.length()

    /**
     * {@inheritDoc}
     */
    override val stream: InputStream
        @Throws(DataStoreException::class)
        get() {
            try {
                return LazyFileInputStream(file)
            } catch (e: IOException) {
                throw DataStoreException("Error opening input stream of " + file.absolutePath, e)
            }

        }

    /**
     * {@inheritDoc}
     */
    override val lastModified: Long
        get() = file.lastModified()

    init {
        assert(file.isFile)
    }
}

